@@ -1,5 +1,5 @@
 /* x86-specific ELF linker support between ld and bfd.
-   Copyright (C) 2019-2023 Free Software Foundation, Inc.
+   Copyright (C) 2019-2025 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -28,6 +28,14 @@ enum elf_x86_prop_report
   prop_report_shstk	= 1 << 3    /* Report missing SHSTK property.  */
 };
 
+/* ISA level report control.  */
+enum elf_x86_isa_level_report
+{
+  isa_level_report_none	  = 0,		/* Do nothing.  */
+  isa_level_report_needed = 1 << 0,	/* Needed x86-64 ISA level.  */
+  isa_level_report_used	  = 1 << 1 	/* Used x86-64 ISA level.  */
+};
+
 /* Used to pass x86-specific linker options from ld to bfd.  */
 struct elf_linker_x86_params
 {
@@ -61,9 +69,32 @@ struct elf_linker_x86_params
   /* Report relative relocations.  */
   unsigned int report_relative_reloc : 1;
 
+  /* Mark PLT with dynamic tags.  */
+  unsigned int mark_plt : 1;
+
+  /* Add the GLIBC_ABI_GNU2_TLS version dependency if input object files
+     have R_386_TLS_DESC_CALL or R_X86_64_TLSDESC_CALL relocation:
+     0: Disable.
+     1: Enable.
+     2: Auto.  Enable if libc.so has the GLIBC_ABI_GNU2_TLS version.
+   */
+  unsigned int gnu2_tls_version_tag : 2;
+
+  /* Add the GLIBC_ABI_GNU_TLS version dependency if input object files
+     call ___tls_get_addr:
+     0: Disable.
+     1: Enable.
+     2: Auto.  Enable if libc.so has the GLIBC_ABI_GNU_TLS version.
+     This is only used by i386.
+   */
+  unsigned int gnu_tls_version_tag : 2;
+
   /* X86-64 ISA level needed.  */
   unsigned int isa_level;
 
+  /* Report needed and used x86-64 ISA levels.  */
+  enum elf_x86_isa_level_report isa_level_report;
+
   /* Report missing IBT and SHSTK properties.  */
   enum elf_x86_prop_report cet_report;
 
@@ -77,5 +108,5 @@ struct elf_linker_x86_params
   char call_nop_byte;
 };
 
-extern void _bfd_elf_linker_x86_set_options
+extern void bfd_elf_linker_x86_set_options
   (struct bfd_link_info *, struct elf_linker_x86_params *);
