@@ -916,41 +916,16 @@ ihex_set_arch_mach (bfd *abfd,
   return true;
 }
 
-/* Get the size of the headers, for the linker.  */
-
-static int
-ihex_sizeof_headers (bfd *abfd ATTRIBUTE_UNUSED,
-		     struct bfd_link_info *info ATTRIBUTE_UNUSED)
-{
-  return 0;
-}
-
 /* Some random definitions for the target vector.  */
 
 #define	ihex_close_and_cleanup			  _bfd_generic_close_and_cleanup
 #define ihex_bfd_free_cached_info		  _bfd_generic_bfd_free_cached_info
 #define ihex_new_section_hook			  _bfd_generic_new_section_hook
-#define ihex_get_symtab_upper_bound		  _bfd_long_bfd_0
-#define ihex_canonicalize_symtab		  _bfd_nosymbols_canonicalize_symtab
-#define ihex_make_empty_symbol			  _bfd_generic_make_empty_symbol
-#define ihex_print_symbol			  _bfd_nosymbols_print_symbol
-#define ihex_get_symbol_info			  _bfd_nosymbols_get_symbol_info
-#define ihex_get_symbol_version_string		  _bfd_nosymbols_get_symbol_version_string
-#define ihex_bfd_is_target_special_symbol	  _bfd_bool_bfd_asymbol_false
-#define ihex_bfd_is_local_label_name		  _bfd_nosymbols_bfd_is_local_label_name
-#define ihex_get_lineno				  _bfd_nosymbols_get_lineno
-#define ihex_find_nearest_line			  _bfd_nosymbols_find_nearest_line
-#define ihex_find_nearest_line_with_alt		  _bfd_nosymbols_find_nearest_line_with_alt
-#define ihex_find_line				  _bfd_nosymbols_find_line
-#define ihex_find_inliner_info			  _bfd_nosymbols_find_inliner_info
-#define ihex_bfd_make_debug_symbol		  _bfd_nosymbols_bfd_make_debug_symbol
-#define ihex_read_minisymbols			  _bfd_nosymbols_read_minisymbols
-#define ihex_minisymbol_to_symbol		  _bfd_nosymbols_minisymbol_to_symbol
+#define ihex_sizeof_headers			  _bfd_nolink_sizeof_headers
 #define ihex_bfd_get_relocated_section_contents	  bfd_generic_get_relocated_section_contents
 #define ihex_bfd_relax_section			  bfd_generic_relax_section
 #define ihex_bfd_gc_sections			  bfd_generic_gc_sections
 #define ihex_bfd_lookup_section_flags		  bfd_generic_lookup_section_flags
-#define ihex_bfd_merge_sections			  bfd_generic_merge_sections
 #define ihex_bfd_is_group_section		  bfd_generic_is_group_section
 #define ihex_bfd_group_name			  bfd_generic_group_name
 #define ihex_bfd_discard_group			  bfd_generic_discard_group
@@ -981,6 +956,7 @@ const bfd_target ihex_vec =
   16,				/* AR_max_namelen.  */
   0,				/* match priority.  */
   TARGET_KEEP_UNUSED_SECTION_SYMBOLS, /* keep unused section symbols.  */
+  TARGET_MERGE_SECTIONS,
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
   bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Data.  */
@@ -1011,7 +987,7 @@ const bfd_target ihex_vec =
   BFD_JUMP_TABLE_COPY (_bfd_generic),
   BFD_JUMP_TABLE_CORE (_bfd_nocore),
   BFD_JUMP_TABLE_ARCHIVE (_bfd_noarchive),
-  BFD_JUMP_TABLE_SYMBOLS (ihex),
+  BFD_JUMP_TABLE_SYMBOLS (_bfd_nosymbols),
   BFD_JUMP_TABLE_RELOCS (_bfd_norelocs),
   BFD_JUMP_TABLE_WRITE (ihex),
   BFD_JUMP_TABLE_LINK (ihex),
