@@ -35,9 +35,23 @@ check()
     fi
 }
 
+check_missing()
+{
+    if grep -q "$2" "$1"
+    then
+	echo "Found unexpected symbol in $1:"
+	echo "   $2"
+	echo ""
+	echo "Actual output below:"
+	cat "$1"
+	exit 1
+    fi
+}
+
 check ver_test_14.syms "V1 *\(0x[0-9a-f][048c]\)\? t2()$"
 check ver_test_14.syms "V1 *\(0x[0-9a-f][048c]\)\? t3()$"
 check ver_test_14.syms "V1 *\(0x[0-9a-f][048c]\)\? t4()$"
-check ver_test_14.syms "Base *\(0x[0-9a-f][048c]\)\? t4_2a$"
+check ver_test_14.syms "t4_2a$"
+check_missing ver_test_14.syms "Base *\(0x[0-9a-f][048c]\)\? t4_2a$"
 
 exit 0
