@@ -755,6 +755,8 @@
 					/*   Note: name must be "LINUX".  */
 #define NT_ARM_ZT       0x40d           /* AArch64 SME2 ZT registers.  */
 					/*   Note: name must be "LINUX".  */
+#define NT_ARM_FPMR     0x40e           /* AArch64 FPMR.  */
+					/*   Note: name must be "LINUX".  */
 #define NT_ARM_GCS	0x410		/* AArch64 Guarded Control Stack
 					   registers.  */
 					/*   Note  name must be "LINUX".  */
@@ -1377,6 +1379,11 @@
 
 #define VERSYM_HIDDEN		0x8000
 
+/* This flag appears in a Versym structure.  It means that the symbol
+   is the base version.  */
+
+#define VERSYM_BASE		0x0001
+
 /* This is the mask for the rest of the Versym information.  */
 
 #define VERSYM_VERSION		0x7fff
