@@ -2,7 +2,7 @@
 tmpdir/arm-app.*:     file format elf32-(little|big)arm.*
 
 DYNAMIC RELOCATION RECORDS
-OFFSET   TYPE              VALUE 
+OFFSET +TYPE +VALUE
 .* R_ARM_COPY        data_obj
 .* R_ARM_JUMP_SLOT   lib_func1
 
